<?php
/**
* Simple Autoloader
* Attempts to load classes from base directory
* Replaces separators with directory separators
* 
*/
class AutoLoader {
	const NAMESPACE_SEPARATOR = '_';
 	const FILE_SUFFIX = '.php';
 	
 	private $baseDir;
 	private static $instance;
	
 	public function __construct() {
 		$this->baseDir = dirname(__DIR__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR;
 	}
 	/**
 	* Registers the autoloader to load the classes
 	* @uses AutoLoader::load()
 	*/ 
 	public static function register() {
 		if(self::$instance) {
 			return self::$instance;
 		}
 		$className = __CLASS__;
 		self::$instance = new $className;
 		spl_autoload_register(array(self::$instance, 'load'));
 	} 	

 	/**
 	 * Actual loading of the class
 	 * @param string $className
 	 */
 	protected function load($className) {
 		$fileName = $this->baseDir
 			. str_replace(self::NAMESPACE_SEPARATOR, DIRECTORY_SEPARATOR, $className)
 			. self::FILE_SUFFIX;
 		if(!file_exists($fileName)) {
 			return;
 		}
 		include ($fileName);
 	}
 	
	/**
	 * Add include path
	 * @param unknown_type $path include path
	 */
	public static function add_include_path($path) {
		set_include_path($path.PATH_SEPARATOR.get_include_path());
	} 	
}