<?php
/**
 * Abstract wrapper to DB Iterator
 * to convert DB rows to structured entities
 */
abstract class Db_Iterator_Converter implements Iterator, Countable {
	/**
	 * @var Db_Iterator
	 */
	private $rows;
	public function __construct(Db_Iterator $rows) {
		$this->rows = $rows;
	}
	public function current() {
		return $this->convert_row($this->rows->current());
	}
	/** (non-PHPdoc)
	 * @see Iterator::next()
	 */
	public function next() {
		return $this->rows->next();
	}
	/** (non-PHPdoc)
	 * @see Iterator::key()
	 */
	public function key() {
		return $this->rows->key();
	}
	/** (non-PHPdoc)
	 * @see Iterator::valid()
	 */
	public function valid() {
		return $this->rows->valid();
	}
	/** (non-PHPdoc)
	 * @see Iterator::rewind()
	 */
	public function rewind() {
		return $this->rows->rewind();
	}
	/** (non-PHPdoc)
	 * @see Countable::count()
	 */
	public function count() {
		return $this->rows->count();
	}
	/**
	 * Convert row to a structured entity
	 * @param array $row
	 * @return mixed
	 */
	abstract protected function convert_row(array $row);
}
