<?php
/**
 * Administartor page controller
 * @author alex
 *
 */
class Stock_Page_Admin_Controller extends Stock_Controller {

	/**
	 * @var Stock_Page_Admin_View
	 */
	private $view;
	/**
	* @var Stock_Page_Admin_Context
	*/	
	private $context;
	
	public function __construct() {
		$this->context = new Stock_Page_Admin_Context();
		$this->view = new Stock_Page_Admin_View($this->context); 
	}
	
	public function run() {
		if (!SessionManager::instance()->get(self::KEY_ADMIN)) {
			Stock_Router::redirect(Stock_Router::ACT_LOGOUT);
			return;
		}
		
		$commUsers = $this->getUsersComm();
		$totalCommission = $this->calcTotalCommission($commUsers);
		$this->context->setTotalComission($totalCommission);
		
		$usersInfo = $this->buildUsersInfo($commUsers);
		$this->context->setUsersInfo($usersInfo);
		$this->view->show();
	}
	
	private function buildUsersInfo(array $commUsers) {
		$userStorage = new Stock_User_Storage();
		$users = $userStorage->loadAll();
		$result = array();
		/* @var $user Stock_User */
		foreach($users AS $user) {
			$quotesValue = $this->calcQuotesValue($user);
			$result[$user->getId()] = array(
				'id' => $user->getId(),
				'name' => $user->getName(),
				'balance' => $user->getBalance(),
				'commission' => (isset($commUsers[$user->getId()])) ? $commUsers[$user->getId()] : .0,
				'quotes_value' => $quotesValue,
				'profit' => $this->calcProfit($user, $quotesValue)
			);
		}
		return $result;
	}
	
	private function calcQuotesValue(Stock_User $user) {
		$storage = new Stock_User_Quote_Storage();
		$quotes = $storage->loadPurchased($user->getId());
		$quotesValue = .0;
		/* @var $quote Stock_User_Quote */
		foreach ($quotes AS $quote) {
			$quotesValue+=$quote->getPricePaid() * $quote->getQuantity();
		}
		return $quotesValue;
	}
	private function calcProfit(Stock_User $user, $quotesValue) {	
		return ($user->getBalance() + $quotesValue) - Stock_Page_Register_Controller::INITIAL_BALANCE;
	}
	
	private function getUsersComm() {
		$storage = new Stock_Commission_Storage();
		$commissions = $storage->load();
		
		$commUsers = array();
		/* @var $commission Stock_Commission */
		foreach($commissions AS $commission) {
			$userId = $commission->getUserId();
			if (!isset($commUsers[$userId])) {
				$commUsers[$userId]= .0;
			}
			$commUsers[$userId] += (float)$commission->getCommission();
		}
		return $commUsers;
	}
	
	private function calcTotalCommission(array $commUsers) {
		$totalCommission = .0;
		foreach($commUsers AS $userId => $commission) {
			$totalCommission += $commission;
		}
		return $totalCommission;
	}
}

