<?php
/**
 * Today date chooser
 * Accroding to specification user that already did any trade action 
 * cannot login again before the day of trade
 * @author alex
 *
 */
class Stock_Page_User_DateChoose_Controller extends Stock_Page_User_Controller {
	const USER_NOMINTIME = -1;
	const PARAM_DATE = 'date';
	const DOW_SUNDAY = 7;
	const DOW_SATURDAY = 6;
	
	/**
	 * @var Stock_Page_User_Porfolio_View
	 */
	private $view;
	/**
	* @var Stock_Page_User_Porfolio_Context
	*/	
	private $context;
	
	public function __construct() {
		$this->context = new Stock_Page_User_DateChoose_Context();
		$this->view = new Stock_Page_User_DateChoose_View($this->context);
	}
	
	protected function runUser() {
		$this->context->setUser($this->user);

		$storage = new Stock_Quote_Storage();
		$range = $storage->loadDateRange();
		$dates = array();
		if (!$range) {
			trigger_error("Empty range", E_USER_WARNING);
			Stock_Router::redirect(Stock_Router::ACT_LOGIN_FORM);
			return;
		}
		
		$minTime = strtotime($range['min']);
		$maxTime = strtotime($range['max']);
		$minUserTime = $this->getUserMinTime();
		$minUserTime = ( $minUserTime == self::USER_NOMINTIME ) ? $minTime : $minUserTime;
		
		if (isset($_GET[self::PARAM_DATE]) && $_GET[self::PARAM_DATE]) {
			$date = $_GET[self::PARAM_DATE];
			$selectedTime = strtotime($date);
			if ($selectedTime >= $minUserTime) {
				SessionManager::instance()->put(self::KEY_TODAY_DATE, $date);
				Stock_Router::redirect(Stock_Router::ACT_USER_PORFOLIO);
				return;
			}
		}
		
		$skipDates = $storage->loadNoDataDates($minTime, $maxTime);
		$calendars = $this->createCalendars($minTime, $maxTime, $minUserTime, $skipDates);
		$this->context->setCalendars($calendars);

		$this->view->show();
	}
	
	protected function createCalendars($minTime, $maxTime, $minUserTime, array $skipDates) {
		require_once 'lib/calendar/calendar.class.php';
		$calendarRender = new Calendar();
		$calendarRender->mark_today = false;
		$calendarRender->link_days =2; // only highlighted
		$calendarRender->highlighted_dates = array();
		
		$url = $this->context->createUrl(Stock_Router::ACT_USER_SELECTDATES);
		$url.="&".self::PARAM_DATE."=%Y-%m-%d";
		$calendarRender->formatted_link_to = $url;
		$currentTime = $minUserTime;
		$storage = new Stock_Quote_Storage();
		
		
		$skipDates = array_fill_keys($skipDates, true);
				
		while($currentTime < $maxTime) {
 			$dow = date("N", $currentTime);
			$date = date('Y-m-d',$currentTime);
 			if ($dow == self::DOW_SUNDAY || $dow==self::DOW_SATURDAY || isset($skipDates[$date])) {
 				$currentTime = strtotime("+1 day", $currentTime);
 				continue;
 			}
			$calendarRender->highlighted_dates[] = $date;
			$currentTime = strtotime("+1 day", $currentTime);	
		}
		$calendars = array();
		$currentTime = $minTime;
		while($currentTime < $maxTime) {
			$month = date('n', $currentTime);
			$year = date('Y', $currentTime);
			$calendars[] = $calendarRender->output_calendar($year,$month);
			$currentTime = strtotime("+1 month", $currentTime);
		}
		return $calendars;
	}
	
	/**
	 * Detect user available time by its last trade action
	 * @return int 
	 */
	protected function getUserMinTime() {
		$storage = new Stock_User_Quote_Storage();
		$minTime = self::USER_NOMINTIME;
		$lastPurchase = $storage->loadLastPurchase($this->user->getId());
		if ($lastPurchase) {
			$minTime = strtotime("+1 day",$lastPurchase->getTimestamp());
		}
		$lastSell = $storage->loadLastSell($this->user->getId());
		if ($lastSell && $lastSell->getTimestamp() > $minTime) {
			$minTime = strtotime("+1 day",$lastSell->getTimestamp());
		}
		return $minTime;
	}
	
	protected function validateDate() {
		return true;
	}
}
