<?php
/**
 * Output quotes information in JSON format
 * @author alex
 *
 */
class Stock_Page_User_Quotes_Controller extends Stock_Controller {
	const PARAM_SYMBOL="symbol";
	/**
	 * @var string
	 */
	protected $todayDate;
	/**
	 * @var Stock_User
	 */
	protected $user;
	
	public function __construct() {}
	
	public function run() {
		$action = Stock_Router::getAction();
		if (!$this->validate_user() || !$this->validate_date()) {
			$this->output(array());
			return;			
		}
		$data = array();
		if ($action == Stock_Router::ACT_USER_JSON_SYMBOL) {
			$symbol = (isset($_GET[self::PARAM_SYMBOL])) ? $_GET[self::PARAM_SYMBOL] : '';
			if ($symbol) { 
				$histData = $this->loadHistData(array($symbol));
				$data = array_values($histData);
			}		
		} else {
			$userData = $this->loadUserData();
			if ($userData) {
				$histData = $this->loadHistData(array_keys($userData));
				$data = array_merge(array_values($histData) , array_values($userData));
			}
		}		
		$this->output($data);
	}

	/**
	 * Load stock information for past month
	 * @param array $symbols
	 */
	protected function loadHistData(array $symbols) {
		$data = array();
		$storage = new Stock_Quote_Storage();
		$minDate = date("Y-m-d",strtotime("-1 month", strtotime($this->todayDate)));
		$dateQuotes = $storage->loadRange($minDate, $this->todayDate, $symbols);
		foreach($dateQuotes AS $date => $quotes) {
			/* @var $quote Stock_Quote */
			foreach($quotes As $quote) {
				if (!isset($data[$quote->getSymbol()])) {
					$data[$quote->getSymbol()] = array(
						'label' => $quote->getSymbol(). ' - Stock',
						'data' => array()
					);
				}
				$data[$quote->getSymbol()]['data'][] = array( $this->getJsTimestamp($quote->getDate()) , $quote->getPriceClose() );

			}
		}
		return $data;		
	}
	
	protected function loadUserData() {
		$data = array();
		$storage = new Stock_User_Quote_Storage();
		$userQuotes = $storage->loadPurchased($this->user->getId());
		/* @var $userQuote Stock_User_Quote */
		foreach($userQuotes AS $userQuote) {
			
			if (!isset($data[$userQuote->getSymbol()])) {
				$data[$userQuote->getSymbol()] = array(
					'label' => $userQuote->getSymbol() . ' - Bought',
					'data' => array(),
					'lines' => false
				);
			}
			$data[$userQuote->getSymbol()]['data'][] = array( $this->getJsTimestamp($userQuote->getDate()) , $userQuote->getPricePaid() );
		}
		return $data;
	}
	
	protected function getJsTimestamp($date) {
		return strtotime($date." UTC") * 1000;
	}
	
	protected function output(array $data) {
		echo json_encode($data);
		exit;
	}
	
	protected function validate_user() {
		$this->user = SessionManager::instance()->get(self::KEY_USER);
		if (!$this->user) {
			return false;
		}
		return true;
	
	}
	protected function validate_date() {
		$this->todayDate = SessionManager::instance()->get(self::KEY_TODAY_DATE);
		if (!$this->todayDate) {
			return false;
		}
		return true;
	}
	
}

