<?php
/**
 * User sell/buy page controller
 * @author alex
 *
 */
class Stock_Page_User_Trade_Controller extends Stock_Page_User_Controller {
	const PARAM_SYMBOL = 'symbol';
	const PARAM_QUANTITY = 'quantity';
	/**
	 * @var Stock_Page_User_Trade_View
	 */
	private $view;
	/**
	* @var Stock_Page_User_Trade_Context
	*/	
	private $context;
	
	public function __construct() {
		$this->context = new Stock_Page_User_Trade_Context();
		$this->view = new Stock_Page_User_Trade_View($this->context);
	}
	
	protected function runUser() {
		$action = Stock_Router::getAction();
		$this->context->setUser($this->user);

		$symbol = isset($_REQUEST[self::PARAM_SYMBOL]) ? $_REQUEST[self::PARAM_SYMBOL] : '';
		if (!$symbol) {
			Stock_Router::redirect(Stock_Router::ACT_USER_PORFOLIO);
			return;
		}
		
		$storage = new Stock_Quote_Storage();
		$quote = $storage->loadBySymbol($this->todayDate,$symbol);
		if (!$quote) {
			Stock_Router::redirect(Stock_Router::ACT_USER_PORFOLIO);
			return;
		}

		$this->context->setQuote($quote);
		switch($action) {
			case Stock_Router::ACT_USER_BUY:
				$this->context->setType(Stock_Page_User_Trade_Context::TYPE_BUY);
			break;
			case Stock_Router::ACT_USER_BUY_CONFIRM:
				$this->context->setType(Stock_Page_User_Trade_Context::TYPE_BUY);
				$quantity = isset($_POST[self::PARAM_QUANTITY]) ? (int)$_POST[self::PARAM_QUANTITY] : 0;
				$this->context->setQuantity($quantity);
				try {
					if ($this->processBuy($quantity, $quote)) {
						Stock_Router::redirect(Stock_Router::ACT_USER_PORFOLIO);
						return;
					}
				} catch(Stock_Page_User_Trade_Controller_Exception $e) {
					$this->context->setErrorMsg($e->getMessage());
				}
			break;
			case Stock_Router::ACT_USER_SELL_CONFIRM:
				$this->context->setType(Stock_Page_User_Trade_Context::TYPE_SELL);
				$quantity = isset($_POST[self::PARAM_QUANTITY]) ? (int)$_POST[self::PARAM_QUANTITY] : 0;
				try {
					if ($this->processSell($quantity, $quote)) {
						Stock_Router::redirect(Stock_Router::ACT_USER_PORFOLIO);
						return;
					}
				} catch(Stock_Page_User_Trade_Controller_Exception $e) {
					$this->context->setErrorMsg($e->getMessage());
				}
			case Stock_Router::ACT_USER_SELL:
				$this->context->setType(Stock_Page_User_Trade_Context::TYPE_SELL);
				$maxQuantity = $this->getMaxQuantity($symbol);
				$this->context->setMaxQuantity($maxQuantity);
				if (!$maxQuantity) {
					$this->context->setErrorMsg("Quote is not available");
				}
			break;
			default:
				new Stock_Page_User_Trade_Controller_Exception('Unknown action "'.$action.'"');
		}
		
		$this->view->show();
	}
	/**
	 * Buy specific quantity of market quote
	 * @param int $quantity
	 * @param Stock_Quote $quote
	 * @throws Stock_Page_User_Trade_Controller_Exception
	 * @return boolean
	 */
	protected function processBuy($quantity, Stock_Quote $quote) {
		if ($quantity <= 0) {
			throw new Stock_Page_User_Trade_Controller_Exception("Quantity must be a positive number");
		}
		$userQuoteStorage = new Stock_User_Quote_Storage();
		$userStorage = new Stock_User_Storage();
		
		$timestamp = $this->getCurrentTimestamp();
		$userQuote = new Stock_User_Quote(
			$this->user->getId(), $this->todayDate, $quote->getSymbol(), $quote->getPriceClose(), $quantity, $timestamp  
		);
		
		$user = $userStorage->loadById($this->user->getId());
		$pricePaid = $userQuote->getQuantity() * $userQuote->getPricePaid(); 
		$newBalance = $user->getBalance() - $pricePaid; 
		if ($newBalance < 0.00001) {
			throw new Stock_Page_User_Trade_Controller_Exception("You don't have enough money, check your balance.");
		}
		if (!$userQuoteStorage->savePurchased($userQuote)) {
			throw new Stock_Page_User_Trade_Controller_Exception("Cannot complete transaction");
		}
		if (!$userStorage->updateBalance($user->getId(), $newBalance)) {
			throw new Stock_Page_User_Trade_Controller_Exception("Cannot save new balance");
		}
		$this->saveCommission($timestamp, $pricePaid);		
		$user->setBalance($newBalance);
		SessionManager::instance()->put(self::KEY_USER, $user);	
		return true;
	}
	/**
	 * Sell specific quontity of quote
	 * @param int $quantity
	 * @param Stock_Quote $quote
	 * @throws Stock_Page_User_Trade_Controller_Exception
	 */
	protected function processSell($quantity, Stock_Quote $quote) {
		if ($quantity <= 0) {
			throw new Stock_Page_User_Trade_Controller_Exception("Quantity must be a positive number");
		}
		$userQuoteStorage = new Stock_User_Quote_Storage();
		$userStorage = new Stock_User_Storage();
		$quotes = $userQuoteStorage->loadPurchased($this->user->getId(), $quote->getSymbol());
		$updateQuotes = $this->reduceQuantity($quantity, $quotes);
		foreach($updateQuotes AS $userQuote) {
			if (!$userQuoteStorage->savePurchased($userQuote)) {
				throw new Stock_Page_User_Trade_Controller_Exception("Cannot complete transaction");
			}
		}
		$timestamp = $this->getCurrentTimestamp();
		$soldQuote = new Stock_User_Quote(
			$this->user->getId(), $this->todayDate, $quote->getSymbol(), $quote->getPriceClose(), $quantity, $timestamp  
		);
		if (!$userQuoteStorage->saveSold($soldQuote)) {
			throw new Stock_Page_User_Trade_Controller_Exception("Cannot complete transaction");
		}
		$user = $userStorage->loadById($this->user->getId());
		$pricePaid = $soldQuote->getQuantity() * $soldQuote->getPricePaid(); 
		$newBalance = $user->getBalance() + $pricePaid; 
		if (!$userStorage->updateBalance($user->getId(), $newBalance)) {
			throw new Stock_Page_User_Trade_Controller_Exception("Cannot save new balance");
		}
		$this->saveCommission($timestamp, $pricePaid);	
		$user->setBalance($newBalance);
		SessionManager::instance()->put(self::KEY_USER, $user);	
		return true;
	}
	/**
	 * Calc and save commission
	 * @param int $timestamp
	 * @param float $priceTotal - price of deal
	 */
	protected function saveCommission($timestamp, $priceTotal) {
		$value = (1 * $priceTotal) / 100; // 1%;
		$storage = new Stock_Commission_Storage();
		$commission = new Stock_Commission($timestamp, $this->user->getId(), $value);
		$storage->save($commission);
	}
	/**
	 * Convert today date into timestamp
	 * @return int
	 */
	protected function getCurrentTimestamp() {
		$timestamp = strtotime($this->todayDate);
		return mktime(date("H"),  date("i") , date("s") , date("n",$timestamp) ,date("j",$timestamp) ,date("Y",$timestamp));	
	}
	/**
	 * Update quantity of available quotes
	 * @param int $quantity
	 * @param array $quotes
	 * @throws Stock_Page_User_Trade_Controller_Exception
	 * @return array Stock_User_Quote
	 */
	protected function reduceQuantity($quantity,array $quotes) {
		/* @var $userQuote Stock_User_Quote */
		$left = $quantity;
		$updateQuotes = array();
		foreach($quotes AS $userQuote) {
			if ($left > $userQuote->getQuantity()) {
				$newQuantity = 0;
				$left-=$userQuote->getQuantity();
			} else {
				$newQuantity = $userQuote->getQuantity() - $left;
				$left = 0;
			}

			$userQuote->setQuantity($newQuantity);
			$updateQuotes[]=$userQuote;
			if ($left == 0) {
				break;
			}
		}
		if ($left > 0 || !$updateQuotes) {
			throw new Stock_Page_User_Trade_Controller_Exception("You don't have enough quantity of quote to sell");
		}
		return $updateQuotes;
	}
	/**
	 * Get max available quantity of specific symbol
	 * @param string $symbol
	 * @return number
	 */
	protected function getMaxQuantity($symbol) {	
		$userQuoteStorage = new Stock_User_Quote_Storage();
		$userQuotes = $userQuoteStorage->loadPurchased($this->user->getId(), $symbol);
		$maxQuantity = 0;
		/* @var $userQuote Stock_User_Quote */
		foreach($userQuotes AS $userQuote) {
			$maxQuantity+=$userQuote->getQuantity();
		}
		return $maxQuantity;
	}
}

class Stock_Page_User_Trade_Controller_Exception extends Exception {}