<?php
/**
 * Save and Load quotes on market
 * @author alex
 *
 */
class Stock_Quote_Storage {
	const SQL_LOAD = "
		SELECT `date`,symbol,price_open,price_high,price_low,price_close,volume
		FROM stock_data
	";
	const SQL_LOAD_DATERANGE = "SELECT MIN(`date`) AS min,MAX(`date`) AS max FROM stock_data";	
	const SQL_LOAD_NODATA_DATES = "
		SELECT t1.date FROM `dates_lookup` AS t1
		LEFT JOIN stock_data AS t2 ON t1.date=t2.date
		WHERE  
			t1.date >= '%s' AND t1.date <= '%s'
			AND symbol IS NULL
		GROUP BY t1.date
	";
	const SQL_SAVE = "
		REPLACE INTO stock_data	(`date`,symbol,price_open,price_high,price_low,price_close,volume)
		VALUES ('%s','%s','%s','%s','%s','%s','%s')
	";

	public function __construct() {
		Db::acquire();
	}
	/**
	 * Search quotes by their symbol
	 * @param string $date - date to search Y-m-d
	 * @param string $symbol - part of symbol to search
	 * @return array Stock_Quote
	 */
	public function search($date, $symbol) {	
		$sql = self::SQL_LOAD;
		$where = array();
		$where[]=sprintf("(symbol LIKE '%s%%')", mysql_real_escape_string($symbol));
		$where[]=sprintf("(`date` = '%s')", mysql_real_escape_string($date));
		$sql.= " WHERE ".implode(" AND ",$where);
		$result = Db::acquire()->read($sql);
		$quotes = array();
		foreach($result AS $row) {
			$quotes[$row['symbol']] = $this->createQuote($row);
		}
		return $quotes;
	}
	/**
	 * Load quote by its symbol
	 * @param string $date - date Y-m-d
	 * @param string $symbol - symbol to load
	 * @return Stock_Quote
	 */
	public function loadBySymbol($date, $symbol) {
		$quotes = $this->load($date, array($symbol));
		if (!$quotes) {
			return null;
		}
		return current($quotes);
	}
	/**
	 * Load quotes by their symbols
	 * @param string $date - date Y-m-d
	 * @param array $symbols - array of symbols to load (optional)
	 * @return array Stock_Quote
	 */
	public function load($date, array $symbols = array()) {
		$result = $this->loadRange($date, $date, $symbols);
		if (!isset($result[$date])) {
			return array();
		}
		return $result[$date];
	}
	/**
	 * Load quotes information in date range
	 * @param string $dateMin - low range date Y-m-d
	 * @param string $dateMax - max range date Y-m-d
	 * @param array $symbols - array of symbols to load (optional)
	 * @return array Stock_Quote
	 */
	public function loadRange($dateMin, $dateMax, array $symbols = array()) {
		$sql = self::SQL_LOAD;
		$where = array();
		if ($symbols) {
			$where[]=sprintf("(symbol IN (%s))", "'".implode("','",$symbols)."'");
		}
		if ($dateMin == $dateMax) {
			$where[]=sprintf("(`date` = '%s')", mysql_real_escape_string($dateMin));
		} else {
			$where[]=sprintf("(`date` >= '%s' AND `date` <= '%s')",
				mysql_real_escape_string($dateMin),
				mysql_real_escape_string($dateMax)
			);
		}
		if ($where) {
			$sql.= " WHERE ".implode(" AND ",$where);
		}
		$result = Db::acquire()->read($sql);
		if (!$result) {
			return array();
		}
		$quotes = array();
		foreach($result AS $row) {
			$quotes[$row['date']][$row['symbol']] = $this->createQuote($row);
		}
		return $quotes;
	}
	
	/**
	 * Load available range of dates
	 * @return array ( min => 'Y-m-d' , max => 'Y-m-d' )
	 */
	public function loadDateRange() {
		$sql = self::SQL_LOAD_DATERANGE;
		$result = Db::acquire()->query($sql);
		$row = mysql_fetch_assoc($result);
		return $row;
	}

	/**
	 * 
	 * @param int $minTime
	 * @param int $maxTime
	 * @retrun array 'Y-m-d'
	 */
	public function loadNoDataDates($minTime,$maxTime) {
		$sql = sprintf(self::SQL_LOAD_NODATA_DATES,date("Y-m-d",$minTime),date("Y-m-d",$maxTime));
		$result = Db::acquire()->read($sql);
		$rows = array();
		foreach($result AS $row) {
			$rows[] = $row['date'];
		}
		return $rows;
	}
	
	/**
	 * Save quote info
	 * @param Stock_Quote $quote
	 * @return bool
	 */
	public function save(Stock_Quote $quote) {
		$sql = sprintf(self::SQL_SAVE,
			$quote->getDate(),
			mysql_real_escape_string($quote->getSymbol()),
			$quote->getPriceOpen(),
			$quote->getPriceHigh(),
			$quote->getPriceLow(),
			$quote->getPriceClose(),
			$quote->getVolume()
		);
		return Db::acquire()->query($sql);
	}

	/**
	 * Creates quote object from db row
	 * @param array $row
	 * @return Stock_Quote
	 */
	protected function createQuote(array $row) {
		return new Stock_Quote(
			$row['date'],
			$row['symbol'],
			$row['price_open'],
			$row['price_high'],
			$row['price_low'],
			$row['price_close'],
			$row['volume']
		);
	}
}