<?php
/**
 * Routes action to their controllers
 * 
 * @author alex
 *
 */
class Stock_Router {
	const PARAM_ACTION = "action";
	// ACTIONS:
	const ACT_LOGIN_FORM = "login_form";
	const ACT_LOGIN_PROCESS = "login_process";
	const ACT_LOGOUT = "logout";
	const ACT_REGISTER_FORM = "register_form";
	const ACT_REGISTER_PROCESS = "register_process";
	const ACT_WEBSERVICE_INFO = "webservice_info";
	
	const ACT_USER_SELECTDATES = "user_selectdates";	
	const ACT_USER_PORFOLIO = "user_portfolio";
	const ACT_USER_HISTORY = "user_history";
	const ACT_USER_SEARCH_QUOTE = "user_search_quote";
	const ACT_USER_BUY = "user_buy";
	const ACT_USER_SELL = "user_sell";
	const ACT_USER_SYMBOL = "user_symbol";	
	const ACT_USER_BUY_CONFIRM = "user_buy_confirm";
	const ACT_USER_SELL_CONFIRM = "user_sell_confirm";	
	const ACT_USER_JSON_QUOTES = "user_json_quotes";
	const ACT_USER_JSON_SYMBOL = "user_json_symbol";
	
	const ACT_ADMIN_OVERVIEW = "admin_overview";

	const ACT_WEBSERVICE_QUOTES_INFO = "webservice_quotes_info";
	const ACT_WEBSERVICE_USER_QUOTES = "webservice_user_quotes";
	
	/**
	 * Route action to contoller
	 * @param string $action - self::ACT_*
	 * @return Stock_Controller
	 */
	public static function route($action) {
		switch($action) {
			case self::ACT_WEBSERVICE_INFO:
				return new Stock_Page_Webservice_Controller();
			case self::ACT_WEBSERVICE_QUOTES_INFO:
			case self::ACT_WEBSERVICE_USER_QUOTES:
				return new Stock_Webservice_Controller();
			case self::ACT_ADMIN_OVERVIEW:
				return new Stock_Page_Admin_Controller();
			case self::ACT_USER_SELECTDATES:
				return new Stock_Page_User_DateChoose_Controller();
			case self::ACT_USER_PORFOLIO:
			case self::ACT_USER_SEARCH_QUOTE:
				return new Stock_Page_User_Portfolio_Controller();
			case self::ACT_USER_BUY:
			case self::ACT_USER_SELL:
			case self::ACT_USER_BUY_CONFIRM:
			case self::ACT_USER_SELL_CONFIRM:
				return new Stock_Page_User_Trade_Controller();
			case self::ACT_USER_SYMBOL:
				return new Stock_Page_User_Symbol_Controller();
			case self::ACT_USER_HISTORY:
				return new Stock_Page_User_History_Controller();
			case self::ACT_USER_JSON_QUOTES:
			case self::ACT_USER_JSON_SYMBOL:
				return new Stock_Page_User_Quotes_Controller();
			case self::ACT_REGISTER_FORM:
			case self::ACT_REGISTER_PROCESS:
				return new Stock_Page_Register_Controller();
			case self::ACT_LOGIN_FORM:
			case self::ACT_LOGIN_PROCESS:
			case self::ACT_LOGOUT:
			default:
				return new Stock_Page_Login_Controller();
		}
	}
	
	/**
	 * detect current action
	 * @return string $action - defualt ACT_LOGIN_FORM
	 */
	public static function getAction() {
		return (isset($_REQUEST[self::PARAM_ACTION])) ? $_REQUEST[self::PARAM_ACTION] : self::ACT_LOGIN_FORM;
	}
	
	/**
	 * Redirect to action
	 * @param string $action - self::ACT_*
	 */
	public static function redirect($action) {
		$redirectUrl = self::createUrl($action);
		header("Location: ".$redirectUrl);
		exit;
	}
	
	/**
	 * Create url for action
	 * @param string $action
	 * @param array $params
	 * @return string
	 */
	public static function createUrl($action, array $params = array()) {
		$params[Stock_Router::PARAM_ACTION] = $action;
		return "/index.php?".http_build_query($params);
	}
	
	
}