<?php 
/**
 * Data object for daily quote information of user
 * @author alex
 *
 */
class Stock_User_Quote_DayInfo {
	private $symbol;
	private $quantity = 0;
	private $pricePaid = .0;
	private $priceAvg = .0;
	private $lastPrice = .0;
	private $mktPrice = .0;
	private $profit = .0;
	private $percent = .0;
	private $change = .0;
	
	public function __construct($symbol) {
		$this->symbol = (string)$symbol;
	}
	
	/**
	 * Current quote symbol
	 * @return string
	 */
	public function getSymbol() {
		return $this->symbol;
	}

	/**
	 * Total quantity
	 * @return int
	 */
	public function getQuantity() {
		return $this->quantity;
	}

	/**
	 * Total price paid
	 * @return float
	 */
	public function getPricePaid() {
		return $this->pricePaid;
	}

	/**
	 * Average price for this symbol
	 * @return float
	 */
	public function getPriceAvg() {
		return $this->priceAvg;
	}

	/**
	 * Yesterday closed price on market for this symbol
	 * @return float
	 */
	public function getLastPrice() {
		return $this->lastPrice;
	}

	/**
	 * Current market value of all quotes for this sybol
	 * @return float
	 */
	public function getMktPrice() {
		return $this->mktPrice;
	}

	/**
	 * Expected profit
	 * @return float
	 */
	public function getProfit() {
		return $this->profit;
	}

	/**
	 * Expected profit in %
	 * @return float
	 */
	public function getPercent() {
		return $this->percent;
	}

	/**
	 * Change from yesterday
	 * @return float
	 */
	public function getChange() {
		return $this->change;
	}

	/**
	 * 
	 * @param int $quantity
	 */
	public function setQuantity($quantity) {
		$this->quantity = (int)$quantity;
	}
	
	/**
	 * 
	 * @param float $pricePaid
	 */
	public function setPricePaid($pricePaid) {
		$this->pricePaid = (float)$pricePaid;
	}
	
	/**
	 * 
	 * @param float $priceAvg
	 */
	public function setPriceAvg($priceAvg) {
		$this->priceAvg = (float)$priceAvg;
	}
	
	/**
	 * 
	 * @param float $lastPrice
	 */
	public function setLastPrice($lastPrice) {
		$this->lastPrice = (float)$lastPrice;
	}
	
	/**
	 * 
	 * @param float $mktPrice
	 */
	public function setMktPrice($mktPrice) {
		$this->mktPrice = (float)$mktPrice;
	}
	
	/**
	 * 
	 * @param float $profit
	 */
	public function setProfit($profit) {
		$this->profit = (float)$profit;
	}
	
	/**
	 * 
	 * @param float $percent
	 */
	public function setPercent($percent) {
		$this->percent = (float)$percent;
	}
	
	/**
	 * 
	 * @param float $change
	 */
	public function setChange($change) {
		$this->change = (float)$change;
	}
} 