<?
/**
 * 
 * Helper to combine user quotes with todays
 * stok information
 * @author alex
 *
 */
class Stock_User_Quote_Helper {
	
	/**
	 * User today overview
	 * @param string $todayDate
	 * @param Stock_User $user
	 * @return array ($symbol => Stock_User_Quote_DayInfo $info)
	 */
	public function buildQuotesInfo($todayDate, Stock_User $user) {
		$result = $this->buildUserQuotesInfo($user);
		if (!$result) {
			return array();
		}
		$symbols = array_keys($result);

		$storage = new Stock_Quote_Storage();
		$todayQuotes = $storage->load($todayDate, $symbols);
		$yesterdayDate = date("Y-m-d",strtotime("-1 day",strtotime($todayDate)));
		$yesterdayQuotes = $storage->load($yesterdayDate, $symbols);
		/* @var $todayQuote Stock_Quote */
		foreach ($todayQuotes AS $todayQuote) {
			$symbol = $todayQuote->getSymbol();
			
			/* @var $info Stock_User_Quote_DayInfo */
			$info = $result[$symbol];
			
			$quantity = $info->getQuantity();
			$pricePaid = $info->getPricePaid();
			$info->setLastPrice( $todayQuote->getPriceClose() );
			$mktPrice = $quantity * $todayQuote->getPriceClose();

			/* @var $yQuote Stock_Quote */
			$yQuote = (isset($yesterdayQuotes[$symbol])) ? $yesterdayQuotes[$symbol] : null;
			if ($yQuote) {
				$info->setChange( $todayQuote->getPriceClose() - $yQuote->getPriceClose() );
			}

			$info->setMktPrice($mktPrice);
			$info->setProfit($mktPrice - $pricePaid);
			$info->setPercent( abs(100 - ($mktPrice * 100) / $pricePaid) );
		}
		return $result;
	}
	
	/**
	 * 
	 * Load information about user quotes
	 * and prepare array
	 * @param Stock_User $user
	 * @return array ($symbol => Stock_User_Quote_DayInfo $info)
	 */
	protected function buildUserQuotesInfo(Stock_User $user) {
		$storage = new Stock_User_Quote_Storage();
		$userQuotes = $storage->loadPurchased($user->getId());
		if (!$userQuotes) {
			return array();
		}
		$result = array();
		/* @var $userQuote Stock_User_Quote */
		foreach($userQuotes AS $userQuote) {
			$symbol = $userQuote->getSymbol();
			if (!isset($result[$symbol])) {
				$result[$symbol] = new Stock_User_Quote_DayInfo($symbol);
			}
			/* @var $info Stock_User_Quote_DayInfo */
			$info = $result[$symbol];
			$quantity = $userQuote->getQuantity();
			$oldQuantity = $info->getQuantity();
			$newQuantity = $oldQuantity + $quantity;
			$priceAvg = $info->getPriceAvg();
			$priceAvg = ($oldQuantity * $priceAvg + $quantity * $userQuote->getPricePaid()) / $newQuantity;
			$pricePaid = $info->getPricePaid() + $quantity * $userQuote->getPricePaid();
			
			$info->setPricePaid($pricePaid);
			$info->setQuantity($newQuantity);
			$info->setPriceAvg($priceAvg);
		}
		return $result;
	}
	
}