<?php
/**
 * View and Layout
 * @author alex
 *
 */
abstract class Stock_View {
	/**
	 * Context of the view
	 * @var Stock_View_Context
	 */
 	protected $context;
	
 	public function __construct(Stock_View_Context $context) {
 		$this->context = $context; 		
 	}

	/**
	 * @return string - title of rendered page
	 */
 	abstract protected function title();
	/**
	 * Output valid HTML content of rendered page
	 */
 	abstract protected function content();
	/**
	 * Menu links
	 * @return array - array of title => link
	 */
 	abstract protected function links();
 	
	/**
	 * Logged username
	 * @return string
	 */
 	protected function user() {
 		return '';
 	}
 	/**
 	 * Display HTML of page
 	 */
 	public function show() {
 		$links = $this->links();
 		if (!is_array($links)) {
 			$links = array();
 		}
 		$user = $this->user();
 ?>
 <!DOCTYPE html>
 <html>
 	<head>
 		<link rel="stylesheet" href="css/reset.css" />
 		<link rel="stylesheet" href="css/calendar.css" />
 		<link rel="stylesheet" href="css/base.css" />
 		<link rel="stylesheet" href="css/style.css" />
 		
    	<!--[if lte IE 8]><script type="text/javascript" src="js/flot/excanvas.min.js"></script><![endif]--> 
 		<script src="js/jquery-1.6.2.min.js" type="text/javascript"></script>
    	<script type="text/javascript" src="js/flot/jquery.flot.js"></script> 
 		<title><?=$this->title()?></title>
 	</head>
 	<body>
		<div id="header">
	      <div id="header-content" class="container">
	        <ul id="global-navigation">
	 			<? if ($user) { ?>
	 			<li><div class="user-info">Welcome, <?=$user?></div></li>
	 			<? } ?>
	 			<? foreach ($links AS $title => $url) {?>
	 				<? if ($url) {?>
	 					<li><a href="<?=$url?>"><?=$title?></a></li>
	 				<? } else { ?>
	 					 <li><?=$title?></li>
	 				<? } ?>
	 			<? } ?>
	        </ul>
	      </div>
	    </div>
		<div id="content" class="container">
			<div class="box box-shadow">
  				<h2 class="box-header"><?=$this->title()?></h2>
  				<div id="box-content">
  				<?=$this->content()?>
  				<div style="clear:both"></div>
  				</div>
			</div>
		</div>
 	</body>
 </html>
 <?
	}
}