<?php
/**
 * Provide XML response for Webservice
 * @author alex
 *
 */
class Stock_Webservice_Controller extends Stock_Controller {
	const PARAM_USERNAME = 'username';
	const PARAM_PASSWORD = 'password';
	const PARAM_DATE = 'date';
	const PARAM_SYMBOL = 'symbol';
	
	const ERROR_CODE_AUTH = 'auth';
	const ERROR_CODE_DATE = 'date';
	
	/**
	* @var string
	*/
	protected $todayDate;
	/**
	* @var Stock_User
	*/
	protected $user;
	
	/* (non-PHPdoc)
	 * @see Stock_Controller::run()
	 */
	public function run() {
		$action = Stock_Router::getAction();
		if (!$this->validateUser()) {
			$this->outputError(self::ERROR_CODE_AUTH, "User cannot be authentificated");
			return;
		}
		if (!$this->validateDate()) {
			$this->outputError(self::ERROR_CODE_DATE, "Missing or wrong date format (YYYY-MM-DD)");
			return;
		}
		
		switch($action) {
			case Stock_Router::ACT_WEBSERVICE_QUOTES_INFO:
				$dom = $this->quotesInfo(); 
			break;
			case Stock_Router::ACT_WEBSERVICE_USER_QUOTES:
				$dom = $this->quotesUserQuotes();
			break;
			default:
				throw new Exception("Invalid action: ".$action);
		}
		$this->output($dom);
		return;
	}
	
	protected function quotesUserQuotes() {
		$helper = new Stock_User_Quote_Helper();
		$results = $helper->buildQuotesInfo($this->todayDate, $this->user);

		$dom = new DOMDocument("1.0", "UTF-8");
		$response = $dom->createElement("response");
		$response->setAttribute("length", count($results));
		/* @var $info Stock_User_Quote_DayInfo */
		foreach ($results AS $symbol => $info) {
			$symbolTag = $dom->createElement("symbol");
			$symbolTag->setAttribute("name", $symbol);

			$symbolTag->appendChild( $dom->createElement("priceAvg", number_format($info->getPriceAvg(),2)) );
			$symbolTag->appendChild( $dom->createElement("quantity", $info->getQuantity()) );
			$symbolTag->appendChild( $dom->createElement("lastPrice", number_format($info->getLastPrice(),2)) );
			$symbolTag->appendChild( $dom->createElement("change", number_format($info->getChange(),2)) );
			$symbolTag->appendChild( $dom->createElement("mktPrice", number_format($info->getMktPrice(),2)) );
			$symbolTag->appendChild( $dom->createElement("profit", number_format($info->getProfit(),2)) );
			$symbolTag->appendChild( $dom->createElement("percent", number_format($info->getPercent(),2)) );

			$response->appendChild($symbolTag);
		}
		$dom->appendChild($response);
		return $dom;

	}
	
	protected function quotesInfo() {
		$storage = new Stock_Quote_Storage();
		$symbol = (!empty($_GET[self::PARAM_SYMBOL])) ? trim($_GET[self::PARAM_SYMBOL]) : null;
		$results = $storage->search($this->todayDate, $symbol);
		$dom = new DOMDocument("1.0", "UTF-8");
		$response = $dom->createElement("response");
		$response->setAttribute("length", count($results));
		/* @var $quote Stock_Quote */
		foreach ($results AS $symbol => $quote) {
			$symbolTag = $dom->createElement("symbol");
			$symbolTag->setAttribute("name", $symbol);
			$symbolTag->setAttribute("priceClose", $quote->getPriceClose());
			$symbolTag->setAttribute("volume", $quote->getVolume());
			$response->appendChild($symbolTag);
		}
		$dom->appendChild($response);
		return $dom;
	}
	
	/**
	 * Shows Error xml message
	 * @param string $errorCode
	 * @param string $errorMessage
	 */
	protected function outputError($errorCode, $errorMessage) {
		$dom = new DOMDocument("1.0", "UTF-8");
		$response = $dom->createElement("response");
		$response->setAttribute("length", 0);
		$error = $dom->createElement("error", $errorMessage);
		$error->setAttribute("code", $errorCode);
		$dom->appendChild($response);
		$response->appendChild($error);
		$this->output($dom);
	}
	
	protected function validateUser() {
		$pass = (!empty($_GET[self::PARAM_PASSWORD])) ? trim($_GET[self::PARAM_PASSWORD]) : '';
		$uname = (!empty($_GET[self::PARAM_USERNAME])) ? trim($_GET[self::PARAM_USERNAME]) : '';
		
		$this->user = Stock_User_Login::auth($uname, $pass);
		if (!$this->user) {
			return false;
		}
		return true;
	}
	
	protected function validateDate() {
		$date = (!empty($_GET[self::PARAM_DATE])) ? trim($_GET[self::PARAM_DATE]) : '';
		if (!$date) {
			return false;
		}
		if (!preg_match('/^[\d]{4}-[\d]{2}-[\d]{2}$/', $date)) {
			return false;
		}
		$time = strtotime($date);
		if (!$time) {
			return false;
		}
		$this->todayDate = date("Y-m-d",$time);
		return true;
	}
	
	protected function validate_date() {
		$this->todayDate = SessionManager::instance()->get(self::KEY_TODAY_DATE);
		if (!$this->todayDate) {
			return false;
		}
		return true;
	}
	
	/**
	 * Reender xml and output to the browser
	 * @param DOMDocument $dom
	 */
	protected function output(DOMDocument $dom) {
		$xml = simplexml_import_dom($dom);
		header ("Content-Type:text/xml");
		echo $xml->asXML();
		exit;
	}
}